<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\Manager;
use \Exception as Exception;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Response\Message;

class ArqPadraoRetornoController extends CrudWithParameterBag {
    
    const dataSourceVerba              = 'sdx16100_verbaConsumo';
    const dataSourceLocal              = 'sdx16110_localConsumo';
    const dataSourceConsumoProd        = 'sdx16120_consumoProduto';
    protected $parameterBagColumns = array('NRORG');
    protected $crudService;
    protected $arqPadraoService;
    
              
    public function __construct (ParameterBag $parameterBag, Manager $dataSourceManager, $crudService, $utilService, Services\ArqPadraoRetornoService $arqPadraoService) {
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->crudService = $crudService;
        $this->utilService = $utilService;
        $this->arqPadraoService = $arqPadraoService;
    }
    
    public function findVerba(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceVerba;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request, $response);     
    }
   
    public function saveVerba(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceVerba;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteVerba(DTO\Request\Row $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName    = self::dataSourceVerba;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findLocal(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceLocal;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request, $response);     
    }
   
    public function saveLocal(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceLocal;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteLocal(DTO\Request\Row $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName    = self::dataSourceLocal;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findConsumoProd(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator" => "=",
                "value" => $value
            );
        }
            
        // para o CTRL+F funcionar
        for($i = 0; $i < count($conditions); $i++) {
            if(strpos($dataSourceFilter[$i]["columnName"], '|') != false) {
                $dataSourceFilter[$i]["operator"] = "MAPPED_LIKE_ALL";
            }
        }

        $newRequest = new DTO\Request\Filter (
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        $this->dataSourceName      = self::dataSourceConsumoProd;
        $this->parameterBagColumns = array('CDFILIAL', 'NRORG', 'CDCAIXA', 'TCDCAIXA');
        $result = parent::find($newRequest, $response);     
        return $result;
    }
    
    public function buildParams($conditions) {
        if(!isset($conditions['CDFILIAL'])) {
            $conditions['CDFILIAL'] = '';
        }
        
        return $conditions;
    }
   
    public function saveConsumoProd(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceConsumoProd;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteConsumoProd(DTO\Request\Row $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName    = self::dataSourceConsumoProd;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function getProdutosCadastroAutomatico ($row){
        if($row['INTERVALISTA'] == 'L'){
            $product = array();
            foreach($row['CDPRODUTO'] as $key => $prod){
                $newProd['CDPRODUTO'] = $row['CDPRODUTO'][$key];
                $newProd['CDARVPROD'] = $row['CDARVPRODL'][$key];
                $newProd['DSBUTTON']  = $row['NMPRODUTOL'][$key];
                array_push($product, $newProd);
            }
        } else if($row['INTERVALISTA'] == 'I'){
            if(empty($row['CDARVPRODINI']) || empty($row['CDARVPRODFINAL'])) {
                $product = $this->arqPadraoService->getIntervalProdAll($row);
    		} else {
    		    $product = $this->arqPadraoService->getIntervalProd($row);
    		}
        } else if($row['INTERVALISTA'] == 'D'){
             $product = $this->arqPadraoService->getIntervalProdNivel($row);
        }
		return $product;
    }
    
    public function saveConsumoProdAutomatico (DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $prodArray = [];
        
        if (isset($row['CDARVPRODINI']) && isset($row['CDARVPRODFINAL'])) {
    	    $prodArray = $this->arqPadraoService->getIntervalProd($row);
        }
    	
    // 	var_dump($prodArray);
    // 	var_dump($row);
    
        
        // verificar se algum produto ja está cadastrado
        // se tiver devolver uma mensagem para perguntar se deseja manter ou sobrescrever
        $caixaText = [];
        if (isset($row['CDCAIXA']) && is_array($row['CDCAIXA']) && !empty($row['CDCAIXA'])) {
            $caixaText = implode("_", $row['CDCAIXA']);
        }
        
        if (isset($row['CDARVPRODINI']) && isset($row['CDARVPRODFINAL']) && isset($row['CDFILIAL'])) {
            $produtosCadastrados = $this->arqPadraoService->getProdutosCadastrados($row['CDFILIAL'], $caixaText, $row['CDARVPRODINI'], $row['CDARVPRODFINAL']);
            // var_dump($produtosCadastrados);
            
            if (!empty($produtosCadastrados)) {
                $stringProdutos = "";
                foreach ($produtosCadastrados as $produto) {
                    $stringProdutos .= $produto["CDARVPROD"] . " " . $produto["NMPRODUTO"] . ", ";
                }
                $stringProdutos = rtrim($stringProdutos, ", ");
                $response->addMessage(new Message($stringProdutos));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('produtosCadastrados', $produtosCadastrados));
            } 
            else {
                // nao ha produtos cadastrados -> cadastrar tudo
                if (is_array($row['CDCAIXA']) && !empty($row['CDCAIXA'])) {
                    foreach ($row['CDCAIXA'] as $CDCAIXA) {
                        $this->arqPadraoService->insertConsumo($row, $CDCAIXA);
                    }
                }
            }
        }
    }
    
    public function saveConsumoProdAutomaticoManter (DTO\Request $request, DTO\Response $response) {
        // manter -> inserir apenas produtos nao cadastrados
        $row = $request->getRow();
        
        if (isset($row['CDARVPRODINI']) && isset($row['CDARVPRODFINAL']) && isset($row['CDFILIAL']) && isset($row['CDCAIXA']) && isset($row['CDVERBA']) && isset($row['CDLOCAL'])) {
            if (is_array($row['CDCAIXA']) && !empty($row['CDCAIXA'])) {
                foreach ($row['CDCAIXA'] as $CDCAIXA) {
                    $this->arqPadraoService->insertConsumo($row, $CDCAIXA);
                }
            }
        }
    }
    
    public function saveConsumoProdAutomaticoSobrepor (DTO\Request $request, DTO\Response $response) {
        // sobrepor -> deletar os produtos cadastrados e cadastrar novamente
        $row = $request->getRow();

        // delete cadastrados
        if (isset($row['CDARVPRODINI']) && isset($row['CDARVPRODFINAL']) && isset($row['CDFILIAL']) && isset($row['CDCAIXA']) && isset($row['CDVERBA']) && isset($row['CDLOCAL'])) {
            if (is_array($row['CDCAIXA']) && !empty($row['CDCAIXA'])) {
                foreach ($row['CDCAIXA'] as $CDCAIXA) {
                    $this->arqPadraoService->deleteConsumo($row, $CDCAIXA);
                    $this->arqPadraoService->insertConsumo($row, $CDCAIXA);
                }
            }
        }
    }
    
    public function gerarArquivosPadrao (DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();

        $parameters = ['CDLOCAL', 'CDVERBA', 'CDCLIENTE', 'CDCCUSCLIE', 'CDCONSUMIDOR', 'CDPRODUTO'];

        foreach ($parameters as $param) {
            if (!isset($row[$param]) || empty($row[$param])) {
                $row[$param] = '';  // Atribuir valor vazio se for nulo
                $row["T$param"] = 'T';  // Definir o correspondente T_PARAMETRO como 'T'
            } else {
                $row["T$param"] = '';
            }
        }
        
        $arrayFields = ['CDCCUSCLIE', 'CDCONSUMIDOR', 'CDVERBA', 'CDLOCAL', 'CDPRODUTO'];

        foreach ($arrayFields as $field) {
            if (is_array($row[$field]) && !empty($row[$field])) {
                $row[$field] = implode('_', $row[$field]);
            }
        }
        
        $dadosAnalitico = $this->arqPadraoService->buscaDadosAnalitico($row);
        $dadosSintetico = $this->arqPadraoService->buscaDadosSintetico($row);
        
        if ($row['TIPOARQUIVO'] == 'T') {
            $result = self::geraArquivoTexto($dadosAnalitico, $dadosSintetico);
            // devolve o conteudo dos arquivos para o front como dataset
            $response->addDataSet(new DataSet("resultadoArquivo", $result));
        } else {
            $result = self::geraArquivoExcel($dadosAnalitico, $dadosSintetico);
            // devolve o conteudo dos arquivos para o front como dataset
            $response->addDataSet(new DataSet("resultadoArquivo", $result));
        }
    }
    
    public function geraArquivoTexto($dadosAnalitico, $dadosSintetico) {
        // Inicializando o conteúdo para os dois tipos de arquivo
        $conteudos = [
            'analitico' => '',
            'sintetico' => ''
        ];
    
        // Gerando o conteúdo para cada tipo de arquivo a partir dos dados fornecidos
        foreach ($dadosAnalitico as $linha) {
            $conteudos['analitico'] .= $this->formatarLinha($linha, 'analitico') . PHP_EOL;
        }
        foreach ($dadosSintetico as $linha) {
            $conteudos['sintetico'] .= $this->formatarLinha($linha, 'sintetico') . PHP_EOL;
        }
    
        // Preparando os resultados para envio ao frontend
        $resultados = [];
        foreach ($conteudos as $tipo => $conteudo) {
            $dateTime = new \DateTime();
            $formatDateTime = $dateTime->format('dmY_His');
            $fileName = "RETORNOPADRAO_" . ($tipo === 'analitico' ? '' : 'SINTETICO_') . $formatDateTime . ".txt";
    
            // Adicionando os resultados para serem enviados ao frontend
            $resultados[$tipo] = [
                'fileName'    => $fileName,
                'fileContent' => base64_encode($conteudo),  // Envio direto do conteúdo
                'contentType' => 'text/plain'
            ];
        }
    
        // Retorne os resultados
        return $resultados;
    }
    
    private function formatarLinha($linha, $tipo) {
        $linhaFormatada = [];
    
        // Campos para cada tipo de relatório
        $campos = [
            'analitico' => [
                'CDFILIAL', 'NMFILIAL', 'NRINSJURCLIE', 'NMFANTCLIE', 'CDCONSUMIDOR', 'CDIDCONSUMID',
                'NMCONSUMIDOR', 'CDCCUSCLIE', 'NMCCUSCLIE', 'CDTIPOCONS', 'CDPRODUTO', 'NMPRODUTO',
                'QTVENDIDA', 'VRUNIT', 'VRTOTAL', 'CDCAIXA', 'DTVENDA', 'CDTIPORECE', 'NMVERBA', 'NMLOCAL'
            ],
            'sintetico' => [
                'CDFILIAL', 'NMFILIAL', 'NRINSJURCLIE', 'NMFANTCLIE', 'CDCONSUMIDOR', 'CDIDCONSUMID',
                'NMCONSUMIDOR', 'CDCCUSCLIE', 'NMCCUSCLIE', 'CDTIPOCONS', 'QTVENDIDA', 'VRTOTAL', 'NMVERBA', 'NMLOCAL'
            ]
        ];

        // Formatação de cada campo
        foreach ($campos[$tipo] as $campo) {
            switch ($campo) {
                case 'CDFILIAL':
                    $linhaFormatada[] = str_pad($linha[$campo], 4, " ", STR_PAD_RIGHT);
                    break;
                case 'NMFILIAL':
                    $linhaFormatada[] = str_pad($linha[$campo], 30, " ", STR_PAD_RIGHT);
                    break;
                case 'NRINSJURCLIE':
                    $linhaFormatada[] = str_pad($linha[$campo], 20, " ", STR_PAD_RIGHT);
                    break;
                case 'NMFANTCLIE':
                    $linhaFormatada[] = str_pad($linha[$campo], 50, " ", STR_PAD_RIGHT);
                    break;
                case 'CDCONSUMIDOR':
                case 'CDIDCONSUMID':
                    $linhaFormatada[] = str_pad($linha[$campo], 21, " ", STR_PAD_RIGHT);
                    break;
                case 'NMCONSUMIDOR':
                    $linhaFormatada[] = str_pad($linha[$campo], 50, " ", STR_PAD_RIGHT);
                    break;
                case 'CDCCUSCLIE':
                    $linhaFormatada[] = str_pad($linha[$campo], 10, " ", STR_PAD_RIGHT);
                    break;
                case 'NMCCUSCLIE':
                    $linhaFormatada[] = str_pad($linha[$campo], 40, " ", STR_PAD_RIGHT);
                    break;
                case 'CDTIPOCONS':
                    $linhaFormatada[] = str_pad($linha[$campo], 2, " ", STR_PAD_RIGHT);
                    break;
                case 'CDPRODUTO':
                    $linhaFormatada[] = str_pad($linha[$campo], 10, " ", STR_PAD_RIGHT);
                    break;
                case 'NMPRODUTO':
                    $linhaFormatada[] = str_pad($linha[$campo], 50, " ", STR_PAD_RIGHT);
                    break;
                case 'QTVENDIDA':
                case 'VRTOTAL': // Aplica o mesmo formato para QTVENDIDA e VRTOTAL
                    $formattedValue = number_format($linha[$campo], 2, ',', ''); // Formato brasileiro, exemplo
                    $linhaFormatada[] = str_pad($formattedValue, 10, " ", STR_PAD_LEFT);
                    break;
                case 'VRUNIT':
                    $formattedValue = number_format($linha[$campo], 2, ',', '');
                    $linhaFormatada[] = str_pad($formattedValue, 10, " ", STR_PAD_LEFT);
                    break;
                case 'CDCAIXA':
                    $linhaFormatada[] = str_pad($linha[$campo], 3, " ", STR_PAD_RIGHT);
                    break;
                case 'DTVENDA':
                    $formattedDate = date('d/m/Y H:i:s', strtotime($linha[$campo]));
                    $linhaFormatada[] = str_pad($formattedDate, 19, " ", STR_PAD_RIGHT);
                    break;
                case 'CDTIPORECE':
                    $linhaFormatada[] = str_pad($linha[$campo], 3, " ", STR_PAD_RIGHT);
                    break;
                case 'NMVERBA':
                case 'NMLOCAL': // Aplica o mesmo formato para NMVERBA e NMLOCAL
                    $linhaFormatada[] = str_pad($linha[$campo], 40, " ", STR_PAD_RIGHT);
                    break;
            }
        }
    
        return implode(';', $linhaFormatada); // Separar por ponto e vírgula
    }

    public function geraArquivoExcel($dadosAnalitico, $dadosSintetico) {
        // Campos para cada tipo de relatório
        $campos = [
            'analitico' => [
                'CDFILIAL', 'NMFILIAL', 'NRINSJURCLIE', 'NMFANTCLIE', 'CDCONSUMIDOR', 'CDIDCONSUMID',
                'NMCONSUMIDOR', 'CDCCUSCLIE', 'NMCCUSCLIE', 'CDTIPOCONS', 'CDPRODUTO', 'NMPRODUTO',
                'QTVENDIDA', 'VRUNIT', 'VRTOTAL', 'CDCAIXA', 'DTVENDA', 'CDTIPORECE', 'NMVERBA', 'NMLOCAL'
            ],
            'sintetico' => [
                'CDFILIAL', 'NMFILIAL', 'NRINSJURCLIE', 'NMFANTCLIE', 'CDCONSUMIDOR', 'CDIDCONSUMID',
                'NMCONSUMIDOR', 'CDCCUSCLIE', 'NMCCUSCLIE', 'CDTIPOCONS', 'QTVENDIDA', 'VRTOTAL', 'NMVERBA', 'NMLOCAL'
            ]
        ];
    
        // Inicializando o conteúdo para os dois tipos de arquivo
        $conteudos = [
            'analitico' => implode(';', $campos['analitico']) . PHP_EOL,  // Adicionando cabeçalhos
            'sintetico' => implode(';', $campos['sintetico']) . PHP_EOL
        ];
    
        // Gerando o conteúdo para o arquivo analítico
        foreach ($dadosAnalitico as $linha) {
            $linhaCSV = $this->criarLinhaCSV($linha, $campos['analitico']);
            $conteudos['analitico'] .= $linhaCSV . PHP_EOL;
        }
    
        // Gerando o conteúdo para o arquivo sintético
        foreach ($dadosSintetico as $linha) {
            $linhaCSV = $this->criarLinhaCSV($linha, $campos['sintetico']);
            $conteudos['sintetico'] .= $linhaCSV . PHP_EOL;
        }
    
        // Preparando os resultados para envio ao frontend
        $resultados = [];
        $dateTime = new \DateTime();
        $formatDateTime = $dateTime->format('dmY_His');
        foreach (['analitico', 'sintetico'] as $tipo) {
            $fileName = "RETORNOPADRAO_" . ($tipo === 'analitico' ? '' : 'SINTETICO_') . $formatDateTime . ".csv";
            $resultados[$tipo] = [
                'fileName'    => $fileName,
                'fileContent' => base64_encode($conteudos[$tipo]),  // Codificação em base64
                'contentType' => 'text/csv'
            ];
        }
    
        // Retorne os resultados
        return $resultados;
    }
    
    private function criarLinhaCSV($linha, $campos) {
        $linhaCSV = [];
        foreach ($campos as $campo) {
            $valor = isset($linha[$campo]) ? $linha[$campo] : '';
            // Remove espaços extras no início e no final do valor
            $valorLimpo = trim($valor);
            // Substitui as aspas internas (se necessário) e remove as aspas externas
            $linhaCSV[] = str_replace('"', '""', $valorLimpo);
        }
        return implode(';', $linhaCSV);
    }
}
