<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Services;

class LayoutPreview extends CrudWithParameterBag {
    
    protected $util,
              $parameterBag,
              $layoutService;
    
    protected $dataSourceNameArray = array (
        'filterGrupoPreview'                => 'vnd80600_configterminalcaixa_GrupoPreview',
        'filterProdutosPreview'             => 'vnd80600_configterminalcaixa_ProdutosPreview',
        'filterRecebimentoPreview'          => 'vnd80600_configterminalcaixa_RecebimentoPreview',
        'filterGrupoRecebimentoPreview'     => 'vnd80600_configterminalcaixa_GrupoRecebimentoPreview',
        'filterGrupromocPreview'            => 'vnd80600_configterminalcaixa_GrupromocPreview',
        'filterGrupromocProdPreview'        => 'vnd80600_configterminalcaixa_GrupromocProdPreview',
        'filterGrupoProdCombPreview'        => 'vnd80600_configterminalcaixa_GrupoProdCombPreview',
        'filterProdCombPreview'             => 'vnd80600_configterminalcaixa_ProdCombPreview',
        'filterSubGrupoPreview'             => 'vnd80600_configterminalcaixa_SubGrupoPreview'
    );
    
    protected $parameterBagColumnsArray = array (
        'filterGrupoPreview'                =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterProdutosPreview'             =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterRecebimentoPreview'          =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterGrupoRecebimentoPreview'     =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterGrupromocPreview'            =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterGrupromocProdPreview'        =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterGrupoProdCombPreview'        =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterProdCombPreview'             =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA'),
        'filterSubGrupoPreview'             =>  array('NRORG', 'CDFILIAL', 'NRCONFTELA')
    );

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, Util $util, Services\Layout $layoutService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->util = $util;
        $this->layoutService = $layoutService;
    }
    
    public function findLayout (DTO\Request\Filter $request, DTO\Response $response) {
        $Grupo = $this->findGrupoPreview($request, $response);
        $Produto = $this->findProdutosPreview($request, $response);
        $SubGrupo = $this->findSubGrupoPreview($request, $response);
        $Recebimentos = $this->findGrupoRecebimentosPreview($request, $response);
        $Layout = array(
            'Grupo' => $Grupo, 
            'Produto' => $Produto, 
            'Recebimentos' => $Recebimentos, 
            'SubGrupo' => $SubGrupo
        );
    }  
    
    public function findGrupoPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoPreview'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupoPreview'];
        parent::find($request, $response);
    }
    
    public function findProdutosPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProdutosPreview'];
        $conditions = $this->util->getParamsAsArray($request);
        if(isset($conditions['CDGRUPO'])){
            $conditions['TCDGRUPO'] = 'F';
        }else{
            $conditions['TCDGRUPO'] = 'T';
        }
        $return = $this->layoutService->getProductPreview($conditions);
        foreach ($return as &$product) {
           $product['COLOR'] = '#'.$this->intToHexa($product['COLOR']);
           $product['NRCOLORTEXT'] = '#'.$this->intToHexa($product['NRCOLORTEXT']);
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $return));
    }
    
    public function findGrupoRecebimentosPreview($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoRecebimentoPreview'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupoRecebimentoPreview'];
        parent::find($request, $response);
    }
    
    public function findRecebimentosPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterRecebimentoPreview'];
        $conditions = $this->util->getParamsAsArray($request);
        if(isset($conditions['CDGRUPO'])){
            $conditions['TCDGRUPO'] = 'F';
        }else{
            $conditions['TCDGRUPO'] = 'T';
        }
        $return = $this->layoutService->getRecebimentosPreview($conditions);
        foreach ($return as &$product) {
           $product['COLOR'] = '#'.$this->intToHexa($product['COLOR']);
           $product['NRCOLORTEXT'] = '#'.$this->intToHexa($product['NRCOLORTEXT']);
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $return));
    }
    
    public function findGrupromocPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupromocPreview'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupromocPreview'];
        parent::find($request, $response);
    }
    
    public function findGrupromocProdPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupromocProdPreview'];
        $conditions = $this->util->getParamsAsArray($request);
        if(isset($conditions['CDGRUPO'])){
            $conditions['TCDGRUPO'] = 'F';
        }else{
            $conditions['TCDGRUPO'] = 'T';
        }
        $return = $this->layoutService->getGrupromocProdPreview($conditions);
        foreach ($return as &$product) {
           $product['COLOR'] = '#'.$this->intToHexa($product['COLOR']);
           $product['NRCOLORTEXT'] = '#'.$this->intToHexa($product['NRCOLORTEXT']);
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $return));
    }
    
    public function findGrupoProdCombPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupoProdCombPreview'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupoProdCombPreview'];
        parent::find($request, $response);
    }
    
    public function findProdCombPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProdCombPreview'];
        $conditions = $this->util->getParamsAsArray($request);
        if(isset($conditions['CDGRUPO'])){
            $conditions['TCDGRUPO'] = 'F';
        }else{
            $conditions['TCDGRUPO'] = 'T';
        }
        $return = $this->layoutService->getProdCombPreview($conditions);
        foreach ($return as &$product) {
           $product['COLOR'] = '#'.$this->intToHexa($product['COLOR']);
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $return));
    }
    
    public function findSubGrupoPreview ($request, $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterSubGrupoPreview'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterSubGrupoPreview'];
        parent::find($request, $response);
    }

    public function intToHexa($number) {
        $number = intval($number);
        $number = dechex($number);
        $number = str_pad($number, 6, '0', STR_PAD_LEFT);
        $b = substr($number, 0, 2);
        $g = substr($number, 2, 2);
        $r = substr($number, 4, 2);
        $number = $r . $g . $b;
        return $number;
    }
}