<?php 

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services\RelVendaDiaHoraService;
use \Zeedhi\Framework\Util\Functions;

class RelVendaDiaHoraController {
    protected $relVendaDiaHoraService;

    public function __construct($utilService, RelVendaDiaHoraService $relVendaDiaHoraService) {
        $this->utilService = $utilService;
        $this->relVendaDiaHoraService = $relVendaDiaHoraService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if($this->validaConditions($conditions)) {
            $result = $this->relVendaDiaHoraService->find($conditions);
            
            $response->addDataSet(new DataSet("vnd02500_relVendasDiaHora", $result));
        }
        
        return $result;
    }

    public function buildParams($conditions) {
        $result = [];

        if(!Functions::arrayKeyExists('CDFILIAL', $conditions)) {
            $result['P_CDFILIAL']  = '';
            $result['P_TCDFILIAL'] = 'T';
        } else {
            $result['P_CDFILIAL']   = implode('_', $conditions['CDFILIAL']);
            $result['P_CDOPERADOR'] = '';
            $result['P_TCDFILIAL']  = '';
        }
        
        if(!Functions::arrayKeyExists('CDOPERADOR', $conditions)) {
            $result['P_CDOPERADOR']  = '';
        } else {
            $result['P_CDOPERADOR'] = $conditions['CDOPERADOR'];
        }
        
        if(!Functions::arrayKeyExists('CDLOJA', $conditions)) {
            $result['P_CDLOJA']  = '';
            $result['P_TCDLOJA'] = 'T';
        } else {
            $result['P_CDLOJA']  = implode('_', $conditions['CDLOJA']);
            $result['P_TCDLOJA'] = '';
        }

        if(!Functions::arrayKeyExists('CDCAIXA', $conditions)) {
            $result['P_CDCAIXA']  = '';
            $result['P_TCDCAIXA'] = 'T';
        } else {
            $result['P_CDCAIXA']  = implode('_', $conditions['CDCAIXA']);
            $result['P_TCDCAIXA'] = '';
        }
        
        if(Functions::arrayKeyExists('DTPERIODO', $conditions)) {
            $result['P_DTVENDAINI'] = $conditions['DTPERIODO'][0];
            $result['P_DTVENDAFIN'] = $conditions['DTPERIODO'][1];
        }
        
        if(Functions::arrayKeyExists('HRINI', $conditions)) {
            $result['P_HRINI'] = str_replace(['%', ':'], '', $conditions['HRINI']);
        } else {
            $result['P_HRINI'] = '00';
        }
        
        if(Functions::arrayKeyExists('HRFIN', $conditions)) {
            $result['P_HRFIN'] = str_replace(['%', ':'], '', $conditions['HRFIN']);
        } else {
            $result['P_HRFIN'] = '23';
        }
        
        if(Functions::arrayKeyExists('IDIMPHR', $conditions)) {
            $result['P_IDIMPHR'] = str_replace('%', '', $conditions['IDIMPHR']);
        } else {
            $result['P_IDIMPHR'] = 'N';
        }
        
        if(Functions::arrayKeyExists('MINUTOS', $conditions)) {
            $result['P_MINUTOS'] = str_replace('%', '', $conditions['MINUTOS']);
        } else if(Functions::arrayKeyExists('INTERVALO', $conditions)) {
            $result['P_MINUTOS'] = str_replace(['%', ':'], '', $conditions['INTERVALO']);
            
            if($result['P_MINUTOS'] !== "0000") {
                $result['P_IDIMPHR'] = 'S';
            }
        } else {
            $result['P_MINUTOS'] = '0';
        }
        
        if(Functions::arrayKeyExists('IDRELATORIOFIELD', $conditions)) {
            $result['IDRELATORIO'] = $conditions['IDRELATORIOFIELD'];
        } else {
            $result['IDRELATORIO'] = 0;
        }

        if(isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'I') {
            if(empty($conditions['CDARVPRODF'])){
                $result['P_CDARVPRODF'] = $conditions['CDARVPRODI'];
            }
            
            $result['P_CDARVPRODI'] = str_replace("%", "", $conditions['CDARVPRODI']);
            $result['P_CDARVPRODF'] = str_replace("%", "", $conditions['CDARVPRODF']);
            $result['P_CDARVPRODF'] = str_pad($conditions["CDARVPRODF"], 15, 'z');
            $result['P_TCDPRODUTO'] = null;
            $result['P_CDPRODLIST'] = null;
        } else if(isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'L'){
            $result['P_CDPRODLIST'] = implode('_', $conditions['CDPRODUTO']);
            $result['P_TCDPRODUTO'] = null;
            $result['P_CDARVPRODF'] = null;
            $result['P_CDARVPRODI'] = null;
        } else {
            $result['P_TCDPRODUTO'] = 'T';
            $result['P_CDPRODLIST'] = null;
            $result['P_CDARVPRODF'] = null;
            $result['P_CDARVPRODI'] = null;
        }
        
        return $result;
    }

    private function validaConditions($conditions) {
        if((isset($conditions['P_DTVENDAINI'])) && (!empty($conditions['P_DTVENDAINI'])) &&
           (isset($conditions['P_DTVENDAFIN'])) && (!empty($conditions['P_DTVENDAFIN']))) {
            return true;
        } else {
            return false;
        }
    }
}
