<?php

namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DTO\Request\DataSet;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use Geral\Entities\Filial;
use Geral\Entities\Centcust;
use Zeedhi\Framework\DataSource;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use Teknisa\MAN\Factories\CadastramentosAutomaticoFactory;
use PhpOffice\PhpSpreadsheet\Shared\Date;

class CadastramentosAutomaticoService{ 
    
    
    protected $entityManager;
    protected $environment;
    private $titleType;
    protected $columns;
    protected $columnNames; //nome das colunas da planilha
    protected $optionalColumnNames; //nome das colunas da planilha que aceitam null
    protected $nrorg;
    protected $CadastramentosAutomaticoFactory;
    protected $worksheet;
    protected $errosValid = array();
    
    const CD1_LENGTH = 1;
    const CD2_LENGTH = 2;
    const CD3_LENGTH = 3;
    const CD4_LENGTH = 4;
    const CD5_LENGTH = 5;
    const CD6_LENGTH = 6;
    const CD10_LENGTH = 10;
    const CD12_LENGTH = 12;
    const CD14_LENGTH = 14;
    const CD20_LENGTH = 20;
    const CD21_LENGTH = 21;
    const CD30_LENGTH = 30;
    const CD50_LENGTH = 50;
    const CD100_LENGTH = 100;
    
    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager, CadastramentosAutomaticoFactory $CadastramentosAutomaticoFactory) {
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->CadastramentosAutomaticoFactory = $CadastramentosAutomaticoFactory;
    }
    
    
    public function readXls($row){
        $file = $row['FILE'];
        $columnNames = $row['CAMPOS'];
        $sheetsToLoad = array('Sheasdasdsadet1', 'Sheet1', 'Planilha1');
        
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);
        if(!$xlsFile){
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
        }
        $this->worksheet = $xlsFile->getActiveSheet();
        $table = FileReader::readWorksheetTable($this->worksheet, $columnNames);
        
        return $table;
    }
    
    public function formatXls(Table $table){
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        
        $i = 0;
        foreach ($tableRows as &$row) {
            $index = $row['index'];
            $row = $row['content'];
            for($column = 1; $column < $columnsNumber; $column++){
                $columnLetterIndex = Coordinate::stringFromColumnIndex($column);
                $columnName = $columnsReference[$columnLetterIndex];
                $row[$columnName] = self::formatValue($row[$columnName], $columnName, $index, $columnsReference);
            }
            $row["ROWNUM"] = $i;

            $i++;
        }
        return $tableRows;
    }
    
    private function hexToRgb($hex) { 
       $hex = str_replace("#", "", $hex);
      	if(strlen($hex) == 3) {
      		$r = hexdec(substr($hex,0,1).substr($hex,0,1));
      		$g = hexdec(substr($hex,1,1).substr($hex,1,1));
      		$b = hexdec(substr($hex,2,1).substr($hex,2,1));
      	} else {
      		$r = hexdec(substr($hex,0,2));
      		$g = hexdec(substr($hex,2,2));
      		$b = hexdec(substr($hex,4,2));
      	}
      	$rgb = self::RGBToTColor($r, $g, $b);
      	return $rgb;
    }
    
    private function RGBToTColor($r, $g, $b){
        $Tcolor = $r | ($g << 8) | ($b << 16);
		return $Tcolor;
    }
    
    
    private function formatValue($value, $columnName, $index = null, $columnsReference = array()){
        switch($columnName){
            case $columnName == 'CDFILIAL':
                $value = self::leftZeroFill($value, self::CD4_LENGTH);
                break;
            case $columnName == 'NRCOLORBACK':
                $indice = array_search('NRCOLORBACK', $columnsReference);
                $cell = $indice.$index;
                $value = $this->worksheet->getStyle($cell)->getFill()->getStartColor()->getRGB();
                if(!empty($value)){
                    $value = self::hexToRgb($value);
                }
                break;
            case $columnName == 'NRCOLORTEXT':
                $indice = array_search('NRCOLORTEXT', $columnsReference);
                $cell = $indice.$index;
                $value = $this->worksheet->getStyle($cell)->getFill()->getStartColor()->getRGB();
                if(!empty($value)){
                    $value = self::hexToRgb($value);
                }
                break;
            case $columnName == 'CDCAIXACONT':
                if(!empty($value)){
                    $value = self::leftZeroFill($value, self::CD3_LENGTH);
                }else{
                    $value = $value;
                }
                break;
            case $columnName == 'CDSETOR':
                if(!empty($value)){
                    $value = self::leftZeroFill($value, self::CD4_LENGTH);
                }else{
                    $value = $value;
                }
                break;
            case $columnName == 'CDLOJA':
                $value = self::formatLoja($value);
                break;
            case $columnName == 'NRVAGASSALA':
                $value = self::leftZeroFill($value, self::CD3_LENGTH);
                break;
            case $columnName == 'NRCONFTELA':
                $value = self::leftZeroFill($value, self::CD5_LENGTH);
                break;
            case $columnName == 'NRBUTTON':
                if(!empty($value)){
                    $value = self::leftZeroFill($value, self::CD2_LENGTH);
                }else{
                    $value = $value;
                }
                break;
            case $columnName == 'NRFONTETEXT':
                if(!empty($value)){
                    $value = self::leftZeroFill($value, self::CD2_LENGTH);
                }else{
                    $value = $value;
                }
                break;
            default: 
                $value = strval($value);
        }
        return $value;
    }
    
        private function formatData($data){
            $data = date('d/m/Y', Date::excelToTimestamp($data + 1.0));
            return $data;
        }
    
    private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }
    
    private function formatLoja($cdloja){
        if(strlen($cdloja) == 1){
            return self::leftZeroFill($cdloja, self::CD2_LENGTH);
        }else{
            return $cdloja;
        }
    }
    
    public function salveCampos($dataSet){
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        
        if(!empty($dataSet)){
            if($dataSet['TABELA'] == 'GRUPOPER'){
                $result = $this->CadastramentosAutomaticoFactory->insertGrupOperadores($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'GRUPROMOC'){
                $result = $this->CadastramentosAutomaticoFactory->insertGrupoPromocional($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'GRUPOCOR'){
                $result = $this->CadastramentosAutomaticoFactory->insertGrupoObservacao($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'SALA'){
                $result = $this->CadastramentosAutomaticoFactory->insertMesa($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'CAIXA'){
                $result = $this->CadastramentosAutomaticoFactory->insertCaixa($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'ITMENUCONFTE'){
                $result = $this->CadastramentosAutomaticoFactory->insertConfiguracaoGpProdutos($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'ITMENUCONFTEREB'){
                $result = $this->CadastramentosAutomaticoFactory->insertConfiguracaoGpRecebimentos($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'ITEMPRECO'){
                $result = $this->CadastramentosAutomaticoFactory->insertConfiguracaoItemPreco($dataSet['DATA']);
            }
            
            if($result['operacao']){
                if(isset($result['error']) && !empty($result['error'])){
                    $return['error'] = $result['error'];
                }
                return $return;
            }else{
                $return['operacao'] = $result['operacao'];
                $return['error'] = $result['error'];
            }
            
        }else{
            $return['operacao'] = false;
            $return['error'] = "Não dados para inserção na planilha";
        }
        
        return $return;
    }
    
    public function validCampos($dataSet){
        $this->entityManager->beginTransaction();
        $return = array(
            "operacao" => true,
            "errors" => array()
        );
        
        if(!empty($dataSet)){
            if($dataSet['TABELA'] == 'GRUPOPER'){
                $result = $this->validGrupoOperadores($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'GRUPROMOC'){
                $result = $this->validGrupoPromocionais($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'GRUPOCOR'){
                $result = $this->validGrupoObservacoes($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'SALA'){
                $result = $this->validSala($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'CAIXA'){
                $result = $this->validCaixa($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'ITMENUCONFTE'){
                $result = $this->validConfiguracaoGpProdutos($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'ITMENUCONFTEREB'){
                $result = $this->validConfiguracaoGpRecebimentos($dataSet['DATA']);
            }
            if($dataSet['TABELA'] == 'ITEMPRECO'){
                $result = $this->validConfiguracaoItemPreco($dataSet['DATA'], $dataSet['SUBSCRIBE']);
            }
            
            if(!empty($result)) {
                $this->entityManager->rollback();
            } else {
                $this->entityManager->commit();
            }
            
            $return['errors'] = $result;
            $return['operacao'] = empty($result) ? true : false;
            
        }else{
            $return['operacao'] = false;
            $return['error'] = "Não dados para validar na planilha";
        }
        return $return;
    }
    
    private function getErrorMessage($campo, $value, $tamanho = null, $chave, $alfanumerico = null, $obrigatorio = true){
         
        if($obrigatorio == true && empty($value)){
            $this->errosValid[][$chave] = 'O campo '. $campo .' da linha '.$chave.' encontra-se vazio!';
        }
        if(!empty($value) && $tamanho != null && strlen($value) > $tamanho){
            $this->errosValid[][$chave] = 'O campo '.$campo.' da linha '.$chave.' está com mais de '.$tamanho.' caracteres!';
        }
        if($alfanumerico == true && !is_numeric($value) && !empty($value)){
            $this->errosValid[][$chave] = 'O campo '.$campo.' da linha '.$chave.' contém caracteres alfanumérico!';
        }
    }
    
    public function validGrupoOperadores($data){
        foreach($data as $key => $value){
            $chave = $key + 2;
            self::getErrorMessage('DSGRUPOPER', $value['DSGRUPOPER'], 30, $chave);
            self::getErrorMessage('IDUTCTRSENOP', $value['IDUTCTRSENOP'], null, $chave);
            if(!empty($value['IDUTCTRSENOP']) && !$this->CadastramentosAutomaticoFactory->verificaTipoSimNao($value['IDUTCTRSENOP'])){
                $this->errosValid[][$chave] = 'O campo IDUTCTRSENOP da linha '.$chave.' está com valor inválido. Valores aceitos: Sim ou S, Não ou N!';
            }
        }
        return $this->errosValid;
    }
    
    public function validGrupoPromocionais($data){
        foreach($data as $key => $value){
            $chave = $key + 2;
            self::getErrorMessage('NMGRUPROMOC', $value['NMGRUPROMOC'], 50, $chave);
        }
        return $this->errosValid;
    }
    
    public function validGrupoObservacoes($data){
        foreach($data as $key => $value){
            $chave = $key + 2;
            self::getErrorMessage('DSGRPOCOR', $value['DSGRPOCOR'], 30, $chave);
        }
        return $this->errosValid;
    }
    
    public function validSala($data){
        foreach($data as $key => $value){
            $chave = $key + 2;
            self::getErrorMessage('NMSALA', $value['NMSALA'], 30, $chave);
            self::getErrorMessage('SGSALA', $value['SGSALA'], 15, $chave);
            self::getErrorMessage('NRVAGASSALA', $value['NRVAGASSALA'], 3, $chave, true);
            self::getErrorMessage('CDFILIAL', $value['CDFILIAL'], 4, $chave, true);
            if(!$this->CadastramentosAutomaticoFactory->verificaFilial($value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo CDFILIAL da linha '.$chave.' possui um código de filial/unidade não cadastrado na base!';
            }
        }
        return $this->errosValid;
    }
    
    public function validCaixa($data){
        foreach($data as $key => $value){
            $chave = $key + 2;
            self::getErrorMessage('CDFILIAL', $value['CDFILIAL'], 4, $chave, true);
            if(!$this->CadastramentosAutomaticoFactory->verificaFilial($value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo CDFILIAL da linha '.$chave.' possui um código de filial/unidade não cadastrado na base!';
            }
            
            self::getErrorMessage('CDLOJA', $value['CDLOJA'], 3, $chave, true);
            if(!$this->CadastramentosAutomaticoFactory->verificaLoja($value['CDFILIAL'], $value['CDLOJA'])){
                $this->errosValid[][$chave] = 'O campo CDLOJA da linha '.$chave.' possui um código de loja não cadastrado na base!';
            }
            
            self::getErrorMessage('NMCAIXA', $value['NMCAIXA'], 30, $chave);
            
            self::getErrorMessage('IDCONFVENDA', $value['IDCONFVENDA'], null, $chave);
            if(!$this->CadastramentosAutomaticoFactory->verificaTipoCaixa($value['IDCONFVENDA'])){
                $this->errosValid[][$chave] = 'O campo IDCONFVENDA da linha '.$chave.' está com valor inválido. Valores aceitos: Venda ou 1, Controle ou 2!';
            }
            
            self::getErrorMessage('IDCAIXAFISCA', $value['IDCAIXAFISCA'], null, $chave);
            if(!$this->CadastramentosAutomaticoFactory->verificaHabilitadoCaixa($value['IDCAIXAFISCA'])){
                $this->errosValid[][$chave] = 'O campo IDCAIXAFISCA da linha '.$chave.' está com valor inválido. Valores aceitos: Consumidor ou S, Consumidor/Cliente ou N!';
            }
            
            self::getErrorMessage('IDHABCAIXAVENDA', $value['IDHABCAIXAVENDA'], null, $chave);
            if(!$this->CadastramentosAutomaticoFactory->verificaModalidadeCaixa($value['IDHABCAIXAVENDA'])){
                $this->errosValid[][$chave] = 'O campo IDHABCAIXAVENDA da linha '.$chave.' está com valor inválido.';
            }
            
            self::getErrorMessage('IDTPEMISSAOFOS', $value['IDTPEMISSAOFOS'], null, $chave);
            if(!$this->CadastramentosAutomaticoFactory->verificaTipoEmissaoCaixa($value['IDTPEMISSAOFOS'])){
                $this->errosValid[][$chave] = 'O campo IDTPEMISSAOFOS da linha '.$chave.' está com valor inválido. Valores aceitos: Cupom Fiscal ou ECF, NFC-e Consumidor ou FNC e CF-e-SAT ou SAT';
            }
            
            self::getErrorMessage('CDCAIXACONT', $value['CDCAIXACONT'], 3, $chave, true, false);
            if(!empty($value['CDCAIXACONT']) && !$this->CadastramentosAutomaticoFactory->verificaNCaixaExiste($value['CDFILIAL'], $value['CDCAIXACONT'])){
                $this->errosValid[][$chave] = 'O campo CDCAIXACONT da linha '.$chave.' não possui um código cadastrado na base!';
            }
            
            self::getErrorMessage('CDSETOR', $value['CDSETOR'], 4, $chave, true, false);
            if(!empty($value['CDSETOR']) && !$this->CadastramentosAutomaticoFactory->verificaSetor($value['CDSETOR'])){
                $this->errosValid[][$chave] = 'O campo CDSETOR da linha '.$chave.' não possui um código cadastrado na base!';
            }
        }
        return $this->errosValid;
    }
    
    public function validConfiguracaoGpProdutos($data){
         foreach($data as $key => $value){
            $chave = $key + 2;
            self::getErrorMessage('CDFILIAL', $value['CDFILIAL'], 4, $chave, true);
            if(!empty($value['CDFILIAL']) && !$this->CadastramentosAutomaticoFactory->verificaFilial($value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo CDFILIAL da linha '.$chave.' possui um código de filial/unidade não cadastrado na base!';
            }
            
            self::getErrorMessage('NRCONFTELA', $value['NRCONFTELA'], 5, $chave, true);
            if(!empty($value['NRCONFTELA']) && !$this->CadastramentosAutomaticoFactory->verificaNRCONFTELA($value['NRCONFTELA'], $value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo NRCONFTELA da linha '.$chave.' possui um código de Layout não cadastrado na base!';
            }
            
            self::getErrorMessage('NRPGCONFTELA', $value['NRPGCONFTELA'], 2, $chave, true);
            if(!empty($value['NRPGCONFTELA']) && $value['NRPGCONFTELA'] > 10){
                $this->errosValid[][$chave] = 'O campo NRPGCONFTELA da linha '.$chave.' possui um valor de página fora do intervalo de 1 a 10!';
            }
            
            self::getErrorMessage('NRBUTTON', $value['NRBUTTON'], 36, $chave, true);
            if(!empty($value['NRBUTTON']) && $value['NRBUTTON'] > 36){
                $this->errosValid[][$chave] = 'O campo NRBUTTON da linha '.$chave.' possui um valor de página fora do intervalo de 1 a 36!';
            }
            
            self::getErrorMessage('DSBUTTON', $value['DSBUTTON'], 50, $chave);
            self::getErrorMessage('DSBUTTONESPANH', $value['DSBUTTONESPANH'], 50, $chave, null, false);
            self::getErrorMessage('DSBUTTONINGLES', $value['DSBUTTONINGLES'], 50, $chave, null, false);
            
            self::getErrorMessage('IDTPBUTTON', $value['IDTPBUTTON'], null, $chave);
            if(!$this->CadastramentosAutomaticoFactory->verificaTipoButton($value['IDTPBUTTON'])){
                $this->errosValid[][$chave] = 'O campo IDTPBUTTON da linha '.$chave.' está com valor inválido. Valores aceitos: Grupo de Produtos ou 2, Navegador Direita ou 7, Navegador Esquerda ou 8!';
            }
            
            self::getErrorMessage('IDSHOWTAA', $value['IDSHOWTAA'], null, $chave);
            if(!$this->CadastramentosAutomaticoFactory->verificaShowTAA($value['IDSHOWTAA'])){
                $this->errosValid[][$chave] = 'O campo IDSHOWTAA da linha '.$chave.' está com valor inválido. Valores aceitos: Sim ou S, Não ou N!';
            }
            
            self::getErrorMessage('NRFONTETEXT', $value['NRFONTETEXT'], 2, $chave, true, false);
        }
        return $this->errosValid; 
    }
    
    public function validConfiguracaoGpRecebimentos($data){
         foreach($data as $key => $value){
            $chave = $key + 2;
            self::getErrorMessage('CDFILIAL', $value['CDFILIAL'], 4, $chave, true);
            if(!empty($value['CDFILIAL']) && !$this->CadastramentosAutomaticoFactory->verificaFilial($value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo CDFILIAL da linha '.$chave.' possui um código de filial/unidade não cadastrado na base!';
            }
            
            self::getErrorMessage('NRCONFTELA', $value['NRCONFTELA'], 5, $chave, true);
            if(!empty($value['NRCONFTELA']) && !$this->CadastramentosAutomaticoFactory->verificaNRCONFTELA($value['NRCONFTELA'], $value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo NRCONFTELA da linha '.$chave.' possui um código de Layout não cadastrado na base!';
            }
            
            self::getErrorMessage('NRBUTTON', $value['NRBUTTON'], 2, $chave, true);
            if(!empty($value['NRBUTTON']) && $value['NRBUTTON'] > 6){
                $this->errosValid[][$chave]  = 'O campo NRBUTTON da linha '.$chave.' possui um valor de página fora do intervalo de 1 a 6!';
            }
            
            self::getErrorMessage('DSBUTTON', $value['DSBUTTON'], 50, $chave);
            self::getErrorMessage('NRFONTETEXT', $value['NRFONTETEXT'], 2, $chave, true, false);
        }
    }
    
    private function getErrorMessageItemPreco($campo, $value, $tamanho, $chave, $obrigatorio, $verificaMenor, $numeros, $letras){
         
        if($obrigatorio == true && empty($value)){
            $this->errosValid[][$chave] = 'O campo '. $campo .' da linha '.$chave.' encontra-se vazio!';
        }
        if(!empty($value) && $tamanho != null && strlen($value) > $tamanho){
            $this->errosValid[][$chave] = 'O campo '.$campo.' da linha '.$chave.' está com mais de '.$tamanho.' caracteres!';
        }
        if(!empty($value) && $tamanho != null && $verificaMenor == true && strlen($value) < $tamanho){
            $this->errosValid[][$chave] = 'O campo '.$campo.' da linha '.$chave.' está com menos de '.$tamanho.' caracteres!';
        }
        if($numeros == true && !empty($value) && !is_numeric($value)){
            $this->errosValid[][$chave] = 'O campo '.$campo.' da linha '.$chave.' contém caracteres alfanumérico!';
        }
        if($letras == true && !empty($value) && is_numeric($value)){
            $this->errosValid[][$chave] = 'O campo '.$campo.' da linha '.$chave.' contém caracteres numéricos!';
        }
    }
        
    public function validConfiguracaoItemPreco($data, $subscribe){
         $operacao = false;
         foreach($data as $key => $value){
            $chave = $key + 2;
            $dataValida = true;

            self::getErrorMessageItemPreco('CDFILIAL', $value['CDFILIAL'], 4, $chave, true, true, false, false);
            if(!empty($value['CDFILIAL']) && !$this->CadastramentosAutomaticoFactory->verificaFilial($value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo CDFILIAL da linha '.$chave.' possui um código de filial/unidade não cadastrado na base!';
            }
            
            self::getErrorMessageItemPreco('CDTABEPREC', $value['CDTABEPREC'], 3, $chave, true, true, false, false);
            if(!empty($value['CDTABEPREC']) && !$this->CadastramentosAutomaticoFactory->verificaCDTABEPREC($value['CDTABEPREC'], $value['CDFILIAL'])){
                $this->errosValid[][$chave] = 'O campo CDTABEPREC da linha '.$chave.' possui um código de tabela não cadastrado na base!';
            }
            
            if(!empty($value['DTINIVGPREC'])){
                $valores = explode('/', $value['DTINIVGPREC']);
                
                if(count($valores) !== 3) {
                    $dataValida = false;
                    $this->errosValid[][$chave] = 'O campo DTINIVGPREC da linha '.$chave.' possui um formato de data incorreto!';
                } else if (!checkdate($valores[1], $valores[0], $valores[2])) {
                    $dataValida = false;
                    $this->errosValid[][$chave] = 'O campo DTINIVGPREC da linha '.$chave.' possui um formato de data incorreto!';
                }
            }

            self::getErrorMessageItemPreco('DTINIVGPREC', $value['DTINIVGPREC'], 10, $chave, true, true, false, false);
            if($dataValida) {
                if(!empty($value['DTINIVGPREC']) && !$this->CadastramentosAutomaticoFactory->verificaDTINIVGPREC($value['CDTABEPREC'], $value['CDFILIAL'], $value['DTINIVGPREC'])){
                    $this->errosValid[][$chave] = 'O campo DTINIVGPREC da linha '.$chave.' possui uma vigência não cadastrada na base!';
                }
            }
            
            if(!empty($value['CDPRODUTO']) && !$this->CadastramentosAutomaticoFactory->verificaPRODUTO($value['CDPRODUTO'])){
                $this->errosValid[][$chave] = 'O campo CDPRODUTO da linha '.$chave.' é inexistente na base!';
            }
            
            if(!empty($value['CDPRPAITABPR']) && !$this->CadastramentosAutomaticoFactory->verificaCDPRPAITABPR($value['CDPRPAITABPR'])){
                $this->errosValid[][$chave] = 'O campo CDPRPAITABPR da linha '.$chave.' é inexistente na base!';
            }
            
            self::getErrorMessageItemPreco('CDPRODUTO', $value['CDPRODUTO'], 10, $chave, true, true, false, false);
            if($dataValida) {
                if(!empty($value['CDPRODUTO']) && $this->CadastramentosAutomaticoFactory->verificaCDPRODUTO($value['CDTABEPREC'], $value['CDFILIAL'], $value['DTINIVGPREC'], $value['CDPRODUTO'])){
                    if($subscribe) {
                        $this->CadastramentosAutomaticoFactory->subscribeProduct($value['CDTABEPREC'], $value['CDFILIAL'], $value['DTINIVGPREC'], $value['CDPRODUTO']);
                    } else {
                        $this->errosValid[][$chave] = 'O campo CDPRODUTO da linha '.$chave.' possui um código já cadastrado na base!';
                    }
                }
            }
            
            if(!empty($value['VRPRECITEM'])){
                if(mb_strpos($value['VRPRECITEM'], ',') && !mb_strpos($value['VRPRECITEM'], '.')) {
                    $valores = explode(",", $value['VRPRECITEM']);
                    foreach($valores as $val) {
                        if(!is_numeric($val)) {
                           $this->errosValid[][$chave] = 'O campo VRPRECITEM da linha '.$chave.' possui caracteres inválidos!'; 
                        }
                    }
                } else if(mb_strpos($value['VRPRECITEM'], '.') && !mb_strpos($value['VRPRECITEM'], ',')) {
                    $valores = explode(".", $value['VRPRECITEM']);
                    
                    foreach($valores as $val) {
                        if(!is_numeric($val)) {
                           $this->errosValid[][$chave] = 'O campo VRPRECITEM da linha '.$chave.' possui caracteres inválidos!'; 
                        }
                    }
                } else if(mb_strpos($value['VRPRECITEM'], '.') && mb_strpos($value['VRPRECITEM'], ',')) {
                    $this->errosValid[][$chave] = 'O campo VRPRECITEM da linha '.$chave.' possui um formato de preço incorreto!';
                } else {
                    if(!is_numeric($value['VRPRECITEM'])) {
                           $this->errosValid[][$chave] = 'O campo VRPRECITEM da linha '.$chave.' possui caracteres inválidos!'; 
                    }
                }
            }
            
            if(!empty($value['VRPRECITEMCL'])){
                if(mb_strpos($value['VRPRECITEMCL'], ',') && !mb_strpos($value['VRPRECITEMCL'], '.')) {
                    $valores = explode(",", $value['VRPRECITEMCL']);
                    foreach($valores as $val) {
                        if(!is_numeric($val)) {
                           $this->errosValid[][$chave] = 'O campo VRPRECITEMCL da linha '.$chave.' possui caracteres inválidos!'; 
                        }
                    }
                } else if(mb_strpos($value['VRPRECITEMCL'], '.') && !mb_strpos($value['VRPRECITEMCL'], ',')) {
                    $valores = explode(".", $value['VRPRECITEMCL']);
                    
                    foreach($valores as $val) {
                        if(!is_numeric($val)) {
                           $this->errosValid[][$chave] = 'O campo VRPRECITEMCL da linha '.$chave.' possui caracteres inválidos!'; 
                        }
                    }
                } else if(mb_strpos($value['VRPRECITEMCL'], '.') && mb_strpos($value['VRPRECITEMCL'], ',')) {
                    $this->errosValid[][$chave] = 'O campo VRPRECITEMCL da linha '.$chave.' possui um formato de preço incorreto!';
                } else {
                    if(!is_numeric($value['VRPRECITEMCL'])) {
                           $this->errosValid[][$chave] = 'O campo VRPRECITEMCL da linha '.$chave.' possui caracteres inválidos!'; 
                    }
                }
            }
            
            self::getErrorMessageItemPreco('CDPRPAITABPR', $value['CDPRPAITABPR'], 10, $chave, true, true, false, false);
            self::getErrorMessageItemPreco('VRPRECITEM', $value['VRPRECITEM'], 22, $chave, false, false, false, false);
            self::getErrorMessageItemPreco('VRPRECITEMCL', $value['VRPRECITEMCL'], 22, $chave, false, false, false, false);
            self::getErrorMessageItemPreco('IDPRECVARIA', $value['IDPRECVARIA'], 1, $chave, true, true, false, true);
            self::getErrorMessageItemPreco('IDPRODPROMOC', $value['IDPRODPROMOC'], 1, $chave, true, true, false, true);
            self::getErrorMessageItemPreco('IDEXIPRECOSITE', $value['IDEXIPRECOSITE'], 1, $chave, true, true, false, true);
            self::getErrorMessageItemPreco('NRORG', $value['NRORG'], 22, $chave, true, false, true, false);
            self::getErrorMessageItemPreco('IDATIVO', $value['IDATIVO'], 1, $chave, false, true, false, true);
         }
        return $this->errosValid; 
    }
}